@@ -23,7 +23,7 @@
 ;;; Code:
 (provide 'navi2ch-board)
 (defvar navi2ch-board-ident
-  "$Id: navi2ch-board.el,v 1.44 2002/07/26 13:55:10 nanashi Exp $")
+  "$Id: navi2ch-board.el,v 1.51 2002/11/01 09:48:22 tabmore Exp $")
 
 (eval-when-compile (require 'cl))
 
@@ -56,7 +56,8 @@
 
 (defvar navi2ch-board-regexp-list
   '("^\\([0-9]+\\)\\.dat,\\(.*\\)(\\([0-9]+\\))\n"
-    "^\\([0-9]+\\)\\.dat<>\\(.*\\)[(<]\\([0-9]+\\)[>)]\n"
+    "^\\([0-9]+\\)\\.dat<>\\(.*\\)<\\([0-9]+\\)>\n"
+    "^\\([0-9]+\\)\\.dat<>\\(.*\\)(\\([0-9]+\\))\n"
     "^\\([0-9]+\\)\\.dat,\\(.*\\)$B!J(B\\([0-9]+\\)$B!K(B\n"))
 
 (defvar navi2ch-board-current-board nil)
@@ -178,9 +179,13 @@
   (let ((id (match-string 1))
 	(str (match-string 2))
 	(num (match-string 3)))
-    (setq str (navi2ch-replace-string "^ +" "" str)
-	  str (navi2ch-replace-string " +$" "" str)
-	  str (navi2ch-replace-html-tag str))
+    ;; (setq str (navi2ch-replace-string "^ +" "" str)
+    ;;       str (navi2ch-replace-string " +$" "" str))
+    ;; $B$OCY$$!#(B
+    (when (string-match "^ +" str)
+      (setq str (replace-match "" nil t str)))
+    (when (string-match " +$" str)
+      (setq str (replace-match "" nil t str)))
     (list (cons 'subject str)
 	  (cons 'response num)
 	  (cons 'artid id))))
@@ -192,8 +197,7 @@
 	 (id   (cdr (assq 'id alist)))
 	 board)
     (when id
-      (dolist (x (navi2ch-list-get-board-name-list
-		  navi2ch-list-category-list))
+      (dolist (x navi2ch-list-board-name-list)
 	(when (string= (cdr (assq 'uri x)) uri)
 	  (setq board x)))
       (unless board
@@ -212,6 +216,7 @@
   (when (file-exists-p file)
     (with-temp-buffer
       (navi2ch-insert-file-contents file)
+      (navi2ch-replace-html-tag-with-buffer)
       (goto-char (point-min))
       (let ((regexp (navi2ch-board-regexp-test))
 	    ;; (file-list (directory-files
@@ -353,8 +358,10 @@
       (unless navi2ch-offline
 	(navi2ch-board-save-old-subject-file board)
 	(setq header (navi2ch-board-update-file board))
-	(setq time (or (cdr (assoc "Last-Modified" header))
-		       (cdr (assoc "Date" header))))
+	(setq time (and (not (navi2ch-net-get-state 'not-updated header))
+			(not (navi2ch-net-get-state 'error header))
+			(or (cdr (assoc "Last-Modified" header))
+			    (cdr (assoc "Date" header)))))
 	(when time
 	  (setq board (navi2ch-put-alist 'time time board))))
       (setq navi2ch-board-current-board board)
@@ -575,7 +582,7 @@
 		(setq current-board t)
 		navi2ch-board-current-board)
 	    (navi2ch-board-load-info board)))
-    (setq list (cdr (assq 'bookmark navi2ch-board-current-board)))
+    (setq list (cdr (assq 'bookmark board)))
     (when (and artid (not (member artid list)))
       (setq board (navi2ch-put-alist 'bookmark (cons artid list) board))
       (message "Add bookmark")
