@@ -23,7 +23,7 @@
 ;;; Code:
 (provide 'navi2ch-net)
 (defvar navi2ch-net-ident
-  "$Id: navi2ch-net.el,v 1.53 2002/07/26 13:55:11 nanashi Exp $")
+  "$Id: navi2ch-net.el,v 1.66 2002/12/15 04:23:00 nanashi Exp $")
 
 (eval-when-compile (require 'cl))
 (require 'timezone)
@@ -41,6 +41,13 @@
 (defvar navi2ch-net-status nil)
 (defvar navi2ch-net-header nil)
 (defvar navi2ch-net-content nil)
+(defvar navi2ch-net-state-header-alist
+  '((aborn . "X-Navi2ch-Aborn")		; $B$"$\!<$s$5$l$F$k(B
+    (kako . "X-Navi2ch-Kako")		; $B2a5n%m%0$K$J$C$F$k(B
+    (not-updated . "X-Navi2ch-Not-Updated") ; $B99?7$5$l$F$$$J$$(B
+    (error . "X-Navi2ch-Error"))	; $B%(%i!<(B($B%U%!%$%k$,<hF@$G$-$J$$$H$+(B)
+
+  "STATE $B$N%7%s%\%k$H(B $B<B:]$K%X%C%@$K=q$+$l$kJ8;zNs$N(B alist")
 
 (add-hook 'navi2ch-exit-hook 'navi2ch-net-cleanup)
 
@@ -123,7 +130,9 @@ BODY $B$NI>2ACf$K%(%i!<$,5/$3$k$H(B nil $B$rJV$9!#(B"
 		(delete-process proc))
 	    (setq proc nil))
 	(error (setq proc nil)))
-      (unless proc
+      (when (or (not proc)
+		(not (processp proc))
+		(not (eq (process-status proc) 'open)))
 	(message "now connecting...")
 	(setq proc (open-network-stream navi2ch-net-connection-name
 					buf host port)))
@@ -133,6 +142,7 @@ BODY $B$NI>2ACf$K%(%i!<$,5/$3$k$H(B nil $B$rJV$9!#(B"
 	(erase-buffer))
       (setq navi2ch-net-last-host host)
       (setq navi2ch-net-last-port port)
+      (message "%ssending request..." (current-message))
       (set-process-coding-system proc 'binary 'binary)
       (set-process-sentinel proc 'ignore) ; exited abnormary $B$r=P$5$J$/$9$k(B
       (process-send-string
@@ -161,7 +171,7 @@ BODY $B$NI>2ACf$K%(%i!<$,5/$3$k$H(B nil $B$rJV$9!#(B"
                    (format "Content-length: %d\r\n\r\n%s"
                            (length content) content)
                  "")))
-      (message "%sconnected" (current-message))
+      (message "%sdone" (current-message))
       (navi2ch-net-cleanup-vars)
       (setq navi2ch-net-process proc))))
 
@@ -214,6 +224,13 @@ BODY $B$NI>2ACf$K%(%i!<$,5/$3$k$H(B nil $B$rJV$9!#(B"
 	 (if (looking-at "HTTP/1\\.[01] \\([0-9]+\\)")
 	     (setq navi2ch-net-status (match-string 1)))))))
 
+(defun navi2ch-net-get-protocol (proc)
+  (when (navi2ch-net-get-status proc)
+    (with-current-buffer (process-buffer proc)
+      (goto-char (point-min))
+      (if (looking-at "\\(HTTP/1\\.[01]\\) [0-9]+")
+	  (match-string 1)))))
+
 (defun navi2ch-net-get-header (proc)
   "PROC $B$N@\B3$N%X%C%@It$rJV$9(B"
   (when (navi2ch-net-get-status proc)
@@ -327,9 +344,13 @@ chunk $B$N%5%$%:$rJV$9!#(Bpoint $B$O(B chunk $B$ND>8e$K0\F0!#(B"
 		      (while (and (eq (process-status proc) 'open)
 				  (goto-char (+ p size))
 				  (not (= (point) (+ p size))))
-			(accept-process-output))
+			(accept-process-output proc))
 		      (goto-char (+ p size))))
-		   ((not navi2ch-net-enable-http11)
+		   ((or (string= (navi2ch-net-get-protocol proc)
+				 "HTTP/1.0")
+			(and (stringp (cdr (assoc "Connection" header)))
+			     (string= (cdr (assoc "Connection" header))
+				      "close")))
 		    (while (eq (process-status proc) 'open)
 		      (accept-process-output proc))
 		    (goto-char (point-max))))
@@ -361,11 +382,16 @@ OTHER-HEADER $B$,(B `non-nil' $B$J$i$P%j%/%(%9%H$K$3$N%X%C%@$rDI2C$9$k!#(B
 		     (and navi2ch-net-user-agent
 			  (cons "User-Agent" navi2ch-net-user-agent)))
 	       other-header)))
+       (message "checking file...")
        (setq status (navi2ch-net-get-status proc))
+       (when (and (string= status "416")
+		  (assoc "Range" other-header))
+	 (let ((elt (assoc "Range" other-header)))
+	   (setq other-header (delq elt other-header)
+		 status nil)))
        (unless status
 	 (message "retrying...")
 	 (sit-for 3)))			; $B%j%H%i%$$9$kA0$K$A$g$C$HBT$D(B
-     (message "checking file...")
      (cond ((not (stringp status))
 	    (message "%serror" (current-message))
 	    (setq proc nil))
@@ -393,12 +419,14 @@ OTHER-HEADER $B$,(B `non-nil' $B$J$i$P%j%/%(%9%H$K$3$N%X%C%@$rDI2C$9$k!#(B
 			      other-header)))
 
 
-(defun navi2ch-net-update-file (url file &optional time func location)
+(defun navi2ch-net-update-file (url file &optional time func location diff)
   "FILE $B$r99?7$9$k!#(B
 TIME $B$,(B non-nil $B$J$i$P(B TIME $B$h$j?7$7$$;~$@$199?7$9$k!#(B
 FUNC $B$,(B non-nil $B$J$i$P99?78e(B FUNC $B$r;H$C$F%U%!%$%k$rJQ49$9$k!#(B
 FUNC $B$O(B current-buffer $B$rA`:n$9$k4X?t$G$"$k;v!#(B
-LOCATION $B$,(B non-nil $B$J$i$P(B Location $B%X%C%@$,$"$C$?$i$=$3$K0\F0$9$k$h$&$K$9$k!#(B
+LOCATION $B$,(B non-nil $B$J$i$P(B Location $B%X%C%@$,$"$C$?$i$=$3$K0\F0$9$k$h$&(B
+$B$K$9$k!#(B
+DIFF $B$,(B non-nil $B$J$i$P(B $B:9J,$H$7$F(B FILE $B$r>e=q$-$;$:$KDI2C$9$k!#(B
 $B99?7$G$-$l$P(B header $B$rJV$9(B"
   (let ((dir (file-name-directory file)))
     (unless (file-exists-p dir)
@@ -419,9 +447,13 @@ LOCATION $B$,(B non-nil $B$J$i$P(B Location $B%X%C%@$,$"$C$?$i$=$3$K0\F0$9$
       (cond ((or (not proc)
 		 (not status)
 		 (not header))
-	     (setq header nil))		; $BG0$N$?$a(B
+	     ;; $BG0$N$?$a(B
+	     (setq header (navi2ch-net-add-state 'error header)))
 	    ((string= status "200")
-	     (message "%s: getting new file..." (current-message))
+	     (message (if diff
+			  "%s: getting file diff..."
+			"%s: getting new file...")
+		      (current-message))
 	     (setq cont (navi2ch-net-get-content proc))
 	     (when (and cont func)
 	       (message "%stranslating..." (current-message))
@@ -430,11 +462,15 @@ LOCATION $B$,(B non-nil $B$J$i$P(B Location $B%X%C%@$,$"$C$?$i$=$3$K0\F0$9$
 			    (goto-char (point-min))
 			    (funcall func)
 			    (buffer-string))))
-	     (if cont
+	     (if (and cont (not (string= cont "")))
 		 (with-temp-file file
+		   (when diff
+		     (insert-file-contents file)
+		     (goto-char (point-max)))
 		   (insert cont)
 		   (message "%sdone" (current-message)))
-	       (message "%sfailed" (current-message))))
+	       (setq header (navi2ch-net-add-state 'not-updated header))
+	       (message "%snot updated" (current-message))))
 	    ((and location
 		  (string= status "302")
 		  (assoc "Location" header))
@@ -442,10 +478,10 @@ LOCATION $B$,(B non-nil $B$J$i$P(B Location $B%X%C%@$,$"$C$?$i$=$3$K0\F0$9$
 		   redo t)
 	     (message "%s: redirecting..." (current-message)))
 	    ((string= status "304")
-	     (setq header (cons '("Not-Updated" . "yes")
-				header)))
+	     (setq header (navi2ch-net-add-state 'not-updated header)))
 	    (t
-	     (setq header nil))))	; $B$3$3$KMh$k$O$:$J$$$1$I0l1~(B
+	     ;; $B$3$3$KMh$k$O$:$J$$$1$I0l1~(B
+	     (setq header (navi2ch-net-add-state 'error header)))))
     header))
 
 (defun navi2ch-net-get-length-from-header (header)
@@ -469,30 +505,35 @@ header $B$KD9$5$,4^$^$l$F$$$J$$>l9g$O(B nil $B$rJV$9!#(B"
 (defun navi2ch-net-update-file-diff (url file &optional time)
   "FILE $B$r:9J,$G99?7$9$k!#(B
 TIME $B$,(B `non-nil' $B$J$i$P(B TIME $B$h$j?7$7$$;~$@$199?7$9$k!#(B
-$B99?7$G$-$l$P(B (header state) $B$J(B list $B$rJV$9!#(B
-state $B$O$"$\!<$s$5$l$F$l$P(B aborn $B$H$$$&%7%s%\%k!#(B"
+$B99?7$G$-$l$P(B HEADER $B$rJV$9!#(B"
   (let ((dir (file-name-directory file)))
     (unless (file-exists-p dir)
       (make-directory dir t)))
-  ;; $B%U%!%$%k%5%$%:$HEy$7$$CM$r(B range $B$K$9$k$H%U%!%$%k$rA4ItAw$C$F$/$k(B
-  ;; $B$N$G(B 1- $B$9$k!#(B
-  (let* ((size (max 0 (1- (nth 7 (file-attributes file)))))
-         (proc (navi2ch-net-download-file-range url (format "%d-" size) time)))
-    (if proc
-	(let ((coding-system-for-write 'binary)
-	      (coding-system-for-read 'binary)
-	      (status (navi2ch-net-get-status proc))
-	      (header (navi2ch-net-get-header proc))
-	      cont ret aborn-flag)
-	  (setq aborn-flag (not (navi2ch-net-check-aborn size header)))
-	  (cond (aborn-flag
-		 nil)			; $B$H$j$"$($:2?$b$7$J$$(B
-		((string= status "206")
-		 (message "%s: getting file diff..." (current-message))
-		 (setq cont (navi2ch-net-get-content proc))
-		 (cond ((and (> size 0)
-			     (not (= (aref cont 0) ?\n)))
-			(setq aborn-flag t)) ; \n $B$G;O$^$C$F$J$$>l9g$O$"$\!<$s(B
+  (let* ((coding-system-for-write 'binary)
+	 (coding-system-for-read 'binary)
+	 ;; $B%U%!%$%k%5%$%:$HEy$7$$CM$r(B range $B$K$9$k$H%U%!%$%k$rA4ItAw$C(B
+	 ;; $B$F$/$k$N$G(B 1- $B$9$k!#(B
+	 (size (max 0 (1- (nth 7 (file-attributes file)))))
+	 proc header status aborn-p)
+    (setq proc (navi2ch-net-download-file-range url (format "%d-" size) time))
+    (setq header (and proc
+		      (navi2ch-net-get-header proc)))
+    (setq status (and proc
+		      (navi2ch-net-get-status proc)))
+    (cond ((or (not proc)
+	       (not header)
+	       (not status))
+	   (setq header (navi2ch-net-add-state 'error header)))
+	  ((string= status "304")
+	   (setq header (navi2ch-net-add-state 'not-updated header)))
+	  ((string= status "206")
+	   (if (not (navi2ch-net-check-aborn size header))
+	       (setq aborn-p t)
+	     (message "%s: getting file diff..." (current-message))
+	     (let ((cont (navi2ch-net-get-content proc)))
+	       (cond ((and (> size 0)
+			   (not (= (aref cont 0) ?\n)))
+		      (setq aborn-p t)) ; \n $B$G;O$^$C$F$J$$>l9g$O$"$\!<$s(B
 ;;; 		       ((string= cont "\n")
 ;;; 			(message "%snot updated" (current-message))
 ;;; 			(setq header (cons '("Not-Updated" . "yes")
@@ -503,91 +544,105 @@ state $B$O$"$\!<$s$5$l$F$l$P(B aborn $B$H$$$&%7%s%\%k!#(B"
 			  (insert-file-contents file nil nil size)
 			  (goto-char (point-max))
 			  (insert cont))
-			(message "%sdone" (current-message))
-			(setq ret (list header nil)))))
-		((string= status "200")
-		 (if (not (navi2ch-net-check-aborn size header))
-		     (setq aborn-flag t)
-		   (message "%s: getting whole file..." (current-message))
-		   (with-temp-file file
-		     (insert (navi2ch-net-get-content proc)))
-		   (message "%sdone" (current-message))
-		   (setq ret (list header nil))))
-		((string= status "304")
-		 (setq header (cons '("Not-Updated" . "yes")
-				    header))
-		 (setq ret (list header nil))))
-	  (if (not aborn-flag)
-	      ret
-	    (message "$B$"$\!<$s(B!!!")
-	    (when (and navi2ch-net-save-old-file-when-aborn
-		       (or (not (eq navi2ch-net-save-old-file-when-aborn
-				    'ask))
-			   (y-or-n-p "$B$"$\!<$s(B!!! backup old file? ")))
-	      (copy-file file (read-file-name "file name: ")))
-	    (list (navi2ch-net-update-file url file nil nil) 'aborn)))
-      nil)))
+			(message "%sdone" (current-message)))))))
+	  ((string= status "200")
+	   (if (not (navi2ch-net-check-aborn size header))
+	       (setq aborn-p t)
+	     (message "%s: getting whole file..." (current-message))
+	     (with-temp-file file
+	       (insert (navi2ch-net-get-content proc)))
+	     (message "%sdone" (current-message))))
+	  ((string= status "304")
+	   (setq header (navi2ch-net-add-state 'not-updated header)))
+	  (t
+	   (setq header (navi2ch-net-add-state 'error header))))
+    (if (not aborn-p)
+	header
+      (message "$B$"$\!<$s(B!!!")
+      (when (and navi2ch-net-save-old-file-when-aborn
+		 (or (not (eq navi2ch-net-save-old-file-when-aborn
+			      'ask))
+		     (y-or-n-p "$B$"$\!<$s(B!!! backup old file? ")))
+	(copy-file file (read-file-name "file name: ")))
+      (navi2ch-net-add-state
+       'aborn
+       (navi2ch-net-update-file url file nil nil)))))
 
 (defun navi2ch-net-update-file-with-readcgi (url file &optional time diff)
   "FILE $B$r(B URL $B$+$i(B read.cgi $B$r;H$C$F99?7$9$k!#(B
 TIME $B$,(B non-nil $B$J$i$P(B TIME $B$h$j?7$7$$;~$@$199?7$9$k!#(B
 DIFF $B$,(B non-nil $B$J$i$P:9J,$r<hF@$9$k!#(B
-$B99?7$G$-$l$P(B (header state) $B$J(B list $B$rJV$9!#(B
-state $B$O$"$\!<$s$5$l$F$l$P(B aborn $B$H$$$&%7%s%\%k!#(B"
+$B99?7$G$-$l$P(B HEADER $B$rJV$9!#(B"
   (let ((dir (file-name-directory file))
-	proc header cont)
+	proc header status)
     (unless (file-exists-p dir)
       (make-directory dir t))
     (setq proc (navi2ch-net-download-file url time))
-    (when (and proc
-	       (string= (navi2ch-net-get-status proc) "304"))
-      (setq proc nil))
-    (when proc
-      (let ((coding-system-for-write 'binary)
-	    (coding-system-for-read 'binary))
-	(message "%s: getting file with read.cgi..." (current-message))
-	(setq header (navi2ch-net-get-header proc))
-	(setq cont (navi2ch-net-get-content proc))
-	(if (or (string= cont "")
-		(not cont))
-	    (progn (message "%sfailed" (current-message))
-		   (signal 'navi2ch-update-failed nil))
-	  (message "%sdone" (current-message))
-	  (let (state data cont-size)
-	    (when (string-match "^\\([^ ]+\\) \\(.+\\)\n" cont)
-	      (setq state (match-string 1 cont))
-	      (setq data (match-string 2 cont))
-	      (setq cont (replace-match "" t nil cont)))
-	    (when (and (string-match "\\(OK\\|INCR\\)" state)
-		       (string-match "\\(.+\\)/\\(.+\\)K" data))
-	      (setq cont-size (string-to-number (match-string 1 data))))
-	    (setq cont (navi2ch-string-as-unibyte cont))
-	    (cond
-	     ((string= "+OK" state)
-	      (with-temp-file file
-		(navi2ch-set-buffer-multibyte nil)
-		(when (and (file-exists-p file) diff)
-		  (insert-file-contents file)
-		  (goto-char (point-max)))
-		(insert (substring cont 0 cont-size)))
-	      (list header nil))
-	     ((string= "-INCR" state);; $B$"$\!<$s(B
-	      (with-temp-file file
-		(navi2ch-set-buffer-multibyte nil)
-		(insert (substring cont 0 cont-size))
-		(list header 'aborn)))
-	     ((string= "-ERR" state)
-	      (let ((err-msg (decode-coding-string
-			      data navi2ch-coding-system)))
-		(message "error! %s" err-msg)
-		(cond ((string-match "$B2a5n%m%0AR8K$GH/8+(B" err-msg)
-		       'kako)
+    (setq header (and proc
+		      (navi2ch-net-get-header proc)))
+    (setq status (and proc
+		      (navi2ch-net-get-status proc)))
+    (cond ((or (not proc)
+	       (not header)
+	       (not status))
+	   (setq header (navi2ch-net-add-state 'error header)))
+	  ((string= status "304")
+	   (setq header (navi2ch-net-add-state 'not-updated header)))
+	  ((string= status "200")
+	   (let ((coding-system-for-write 'binary)
+		 (coding-system-for-read 'binary)
+		 cont)
+	     (message (if diff
+			  "%s: getting file diff with read.cgi..."
+			"%s: getting new file with read.cgi...")
+		      (current-message))
+	     (setq cont (navi2ch-net-get-content proc))
+	     (if (or (not cont)
+		     (string= cont ""))
+		 (progn (message "%sfailed" (current-message))
+			(signal 'navi2ch-update-failed nil))
+	       (message "%sdone" (current-message))
+	       (let (state data cont-size)
+		 (when (string-match "^\\([^ ]+\\) \\(.+\\)\n" cont)
+		   (setq state (match-string 1 cont))
+		   (setq data (match-string 2 cont))
+		   (setq cont (replace-match "" t nil cont)))
+		 (when (and (string-match "\\(OK\\|INCR\\)" state)
+			    (string-match "\\(.+\\)/\\(.+\\)K" data))
+		   (setq cont-size (string-to-number (match-string 1 data))))
+		 (setq cont (navi2ch-string-as-unibyte cont))
+		 (cond
+		  ((string= "+OK" state)
+		   (with-temp-file file
+		     (navi2ch-set-buffer-multibyte nil)
+		     (when (and (file-exists-p file) diff)
+		       (insert-file-contents file)
+		       (goto-char (point-max)))
+		     (insert (substring cont 0 cont-size))))
+		  ((string= "-INCR" state);; $B$"$\!<$s(B
+		   (with-temp-file file
+		     (navi2ch-set-buffer-multibyte nil)
+		     (insert (substring cont 0 cont-size)))
+		   (setq header (navi2ch-net-add-state 'aborn header)))
+		  ((string= "-ERR" state)
+		   (let ((err-msg (decode-coding-string
+				   data navi2ch-coding-system)))
+		     (message "error! %s" err-msg)
+		     (cond
+		      ((string-match "$B2a5n%m%0AR8K$GH/8+(B" err-msg)
+		       (setq header (navi2ch-net-add-state 'kako header)))
 ;;; 		      ((and (string-match "html$B2=BT$A(B" err-msg)
 ;;; 			    (string-match "/read\\.cgi/" url))
 ;;; 		       (setq url (replace-match "/offlaw.cgi/" t nil url))
 ;;; 		       (navi2ch-net-update-file-with-readcgi
 ;;; 			url file time diff))
-		      ))))))))))
+		      (t
+		       (setq header
+			     (navi2ch-net-add-state 'error header)))))))))))
+	  (t
+	   ;; $B$3$3$KMh$k$O$:$J$$$1$I0l1~(B
+	   (setq header (navi2ch-net-add-state 'error header))))
+    header))
 
 ;; from Emacs/W3
 (defconst navi2ch-net-url-unreserved-chars
@@ -624,8 +679,14 @@ internet drafts directory for a copy.")
 (defun navi2ch-net-send-message-success-p (proc)
   (let ((str (decode-coding-string (navi2ch-net-get-content proc)
 				   navi2ch-coding-system)))
-    (or (string-match "$B=q$-$3$_$^$7$?!#(B" str)
-	(string-match "$B=q$-$3$_$,=*$o$j$^$7$?!#(B" str))))
+    (cond ((or (string-match "$B=q$-$3$_$^$7$?!#(B" str)
+	       (string-match "$B=q$-$3$_$,=*$o$j$^$7$?!#(B" str))
+	   t)
+	  ((or (string-match "<b>$B%/%C%-!<$,$J$$$+4|8B@Z$l$G$9!*(B</b>" str)
+	       (string-match "<b>$B=q$-$3$_!u%/%C%-!<3NG'(B</b>" str))
+	   'retry)
+	  (t
+	   nil))))
 
 (defun navi2ch-net-send-message-error-string (proc)
   (let ((str (decode-coding-string (navi2ch-net-get-content proc)
@@ -640,8 +701,10 @@ internet drafts directory for a copy.")
   (dolist (pair (navi2ch-net-get-header proc))
     (if (string-equal "Set-Cookie" (car pair))
 	(let ((str (cdr pair)))
-	  (if (string-match "^SPID=\\([^;]+\\);" str)
-	      (return (match-string 1 str)))))))
+	  (cond ((string-match "^SPID=\\([^;]+\\);" str)
+		 (return (match-string 1 str)))
+		((string-match "^PON=\\([^;]+\\);" str)
+		 (return (match-string 1 str))))))))
 
 (defun navi2ch-net-send-message (from mail message subject url referer bbs key spid)
   "$B%a%C%;!<%8$rAw$k!#(B
@@ -715,5 +778,16 @@ internet drafts directory for a copy.")
 	(when (navi2ch-net-update-file url file nil nil t)
 	  file)))))
 
+(defun navi2ch-net-add-state (state header)
+  "HEADER $B$K(B STATE $B$rDI2C$9$k!#(B"
+  (navi2ch-put-alist (cdr (assq state navi2ch-net-state-header-alist))
+		     "yes"
+		     header))
+
+(defun navi2ch-net-get-state (state header)
+  "HEADER $B$+$i(B STATE $B$r<hF@$9$k!#(B"
+  (cdr (assoc (cdr (assq state navi2ch-net-state-header-alist))
+	      header)))
+
 (run-hooks 'navi2ch-net-load-hook)
 ;;; navi2ch-net.el ends here
