@@ -24,7 +24,7 @@
 ;;; Code:
 (provide 'navi2ch-util)
 (defvar navi2ch-util-ident
-  "$Id: navi2ch-util.el,v 1.60 2002/07/26 13:55:11 nanashi Exp $")
+  "$Id: navi2ch-util.el,v 1.74 2002/11/16 08:23:35 tabmore Exp $")
 
 (eval-when-compile (require 'cl))
 (require 'timezone)
@@ -48,8 +48,10 @@
 
 (defvar navi2ch-replace-html-tag-regexp-alist
   '(("<[^<>]+>" . "")
-    ("&#[0-9]+;" . "$B".(B"))
+    ("&[a-z][a-z0-9]*;?" . navi2ch-entity-reference-to-str)
+    ("&#[0-9]+;?" . navi2ch-numeric-reference-to-str))
   "$BCV49$9$k(B html $B$N%?%0$NO"A[%j%9%H(B($B@55,I=8=(B)
+$BCV49@h$,4X?t$@$H!"CV4985$r0z?t$H$7$F$=$N4X?t$r8F$S$@$7$?$b$N$GCV$-BX$($k!#(B
 $B@55,I=8=$,I,MW$J$$>l9g$O(B `navi2ch-replace-html-tag-alist' $B$KF~$l$k(B")
 
 (defvar navi2ch-replace-html-tag-regexp
@@ -59,6 +61,91 @@
   "$BCV49$9$k(B html $B$N%?%0$N@55,I=8=(B
 `navi2ch-replace-html-tag-alist' $B$+$i@8@.$5$l$k(B")
 
+(defvar navi2ch-entity-alist
+  '(("iexcl"    .  161) ("cent"     .  162) ("pound"    .  163)
+    ("curren"   .  164) ("yen"      .  165) ("brvbar"   .  166)
+    ("sect"     .  167) ("uml"      .  168) ("copy"     .  169)
+    ("ordf"     .  170) ("laquo"    .  171) ("not"      .  172)
+    ("shy"      .  173) ("reg"      .  174) ("macr"     .  175)
+    ("deg"      .  176) ("plusmn"   .  177) ("sup2"     .  178)
+    ("sup3"     .  179) ("acute"    .  180) ("micro"    .  181)
+    ("para"     .  182) ("middot"   .  183) ("cedil"    .  184)
+    ("sup1"     .  185) ("ordm"     .  186) ("raquo"    .  187)
+    ("frac14"   .  188) ("frac12"   .  189) ("frac34"   .  190)
+    ("iquest"   .  191) ("Agrave"   .  192) ("Aacute"   .  193)
+    ("Acirc"    .  194) ("Atilde"   .  195) ("Auml"     .  196)
+    ("Aring"    .  197) ("AElig"    .  198) ("Ccedil"   .  199)
+    ("Egrave"   .  200) ("Eacute"   .  201) ("Ecirc"    .  202)
+    ("Euml"     .  203) ("Igrave"   .  204) ("Iacute"   .  205)
+    ("Icirc"    .  206) ("Iuml"     .  207) ("ETH"      .  208)
+    ("Ntilde"   .  209) ("Ograve"   .  210) ("Oacute"   .  211)
+    ("Ocirc"    .  212) ("Otilde"   .  213) ("Ouml"     .  214)
+    ("times"    .  215) ("Oslash"   .  216) ("Ugrave"   .  217)
+    ("Uacute"   .  218) ("Ucirc"    .  219) ("Uuml"     .  220)
+    ("Yacute"   .  221) ("THORN"    .  222) ("szlig"    .  223)
+    ("agrave"   .  224) ("aacute"   .  225) ("acirc"    .  226)
+    ("atilde"   .  227) ("auml"     .  228) ("aring"    .  229)
+    ("aelig"    .  230) ("ccedil"   .  231) ("egrave"   .  232)
+    ("eacute"   .  233) ("ecirc"    .  234) ("euml"     .  235)
+    ("igrave"   .  236) ("iacute"   .  237) ("icirc"    .  238)
+    ("iuml"     .  239) ("eth"      .  240) ("ntilde"   .  241)
+    ("ograve"   .  242) ("oacute"   .  243) ("ocirc"    .  244)
+    ("otilde"   .  245) ("ouml"     .  246) ("divide"   .  247)
+    ("oslash"   .  248) ("ugrave"   .  249) ("uacute"   .  250)
+    ("ucirc"    .  251) ("uuml"     .  252) ("yacute"   .  253)
+    ("thorn"    .  254) ("yuml"     .  255) ("fnof"     .  402)
+    ("Alpha"    .  913) ("Beta"     .  914) ("Gamma"    .  915)
+    ("Delta"    .  916) ("Epsilon"  .  917) ("Zeta"     .  918)
+    ("Eta"      .  919) ("Theta"    .  920) ("Iota"     .  921)
+    ("Kappa"    .  922) ("Lambda"   .  923) ("Mu"       .  924)
+    ("Nu"       .  925) ("Xi"       .  926) ("Omicron"  .  927)
+    ("Pi"       .  928) ("Rho"      .  929) ("Sigma"    .  931)
+    ("Tau"      .  932) ("Upsilon"  .  933) ("Phi"      .  934)
+    ("Chi"      .  935) ("Psi"      .  936) ("Omega"    .  937)
+    ("alpha"    .  945) ("beta"     .  946) ("gamma"    .  947)
+    ("delta"    .  948) ("epsilon"  .  949) ("zeta"     .  950)
+    ("eta"      .  951) ("theta"    .  952) ("iota"     .  953)
+    ("kappa"    .  954) ("lambda"   .  955) ("mu"       .  956)
+    ("nu"       .  957) ("xi"       .  958) ("omicron"  .  959)
+    ("pi"       .  960) ("rho"      .  961) ("sigmaf"   .  962)
+    ("sigma"    .  963) ("tau"      .  964) ("upsilon"  .  965)
+    ("phi"      .  966) ("chi"      .  967) ("psi"      .  968)
+    ("omega"    .  969) ("thetasym" .  977) ("upsih"    .  978)
+    ("piv"      .  982) ("bull"     . 8226) ("hellip"   . 8230)
+    ("prime"    . 8242) ("Prime"    . 8243) ("oline"    . 8254)
+    ("frasl"    . 8260) ("weierp"   . 8472) ("image"    . 8465)
+    ("real"     . 8476) ("trade"    . 8482) ("alefsym"  . 8501)
+    ("larr"     . 8592) ("uarr"     . 8593) ("rarr"     . 8594)
+    ("darr"     . 8595) ("harr"     . 8596) ("crarr"    . 8629)
+    ("lArr"     . 8656) ("uArr"     . 8657) ("rArr"     . 8658)
+    ("dArr"     . 8659) ("hArr"     . 8660) ("forall"   . 8704)
+    ("part"     . 8706) ("exist"    . 8707) ("empty"    . 8709)
+    ("nabla"    . 8711) ("isin"     . 8712) ("notin"    . 8713)
+    ("ni"       . 8715) ("prod"     . 8719) ("sum"      . 8721)
+    ("minus"    . 8722) ("lowast"   . 8727) ("radic"    . 8730)
+    ("prop"     . 8733) ("infin"    . 8734) ("ang"      . 8736)
+    ("and"      . 8743) ("or"       . 8744) ("cap"      . 8745)
+    ("cup"      . 8746) ("int"      . 8747) ("there4"   . 8756)
+    ("sim"      . 8764) ("cong"     . 8773) ("asymp"    . 8776)
+    ("ne"       . 8800) ("equiv"    . 8801) ("le"       . 8804)
+    ("ge"       . 8805) ("sub"      . 8834) ("sup"      . 8835)
+    ("nsub"     . 8836) ("sube"     . 8838) ("supe"     . 8839)
+    ("oplus"    . 8853) ("otimes"   . 8855) ("perp"     . 8869)
+    ("sdot"     . 8901) ("lceil"    . 8968) ("rceil"    . 8969)
+    ("lfloor"   . 8970) ("rfloor"   . 8971) ("lang"     . 9001)
+    ("rang"     . 9002) ("loz"      . 9674) ("spades"   . 9824)
+    ("clubs"    . 9827) ("hearts"   . 9829) ("diams"    . 9830)
+    ("OElig"    .  338) ("oelig"    .  339) ("Scaron"   .  352)
+    ("scaron"   .  353) ("Yuml"     .  376) ("circ"     .  710)
+    ("tilde"    .  732) ("ensp"     . 8194) ("emsp"     . 8195)
+    ("thinsp"   . 8201) ("zwnj"     . 8204) ("zwj"      . 8205)
+    ("lrm"      . 8206) ("rlm"      . 8207) ("ndash"    . 8211)
+    ("mdash"    . 8212) ("lsquo"    . 8216) ("rsquo"    . 8217)
+    ("sbquo"    . 8218) ("ldquo"    . 8220) ("rdquo"    . 8221)
+    ("bdquo"    . 8222) ("dagger"   . 8224) ("Dagger"   . 8225)
+    ("permil"   . 8240) ("lsaquo"   . 8249) ("rsaquo"   . 8250)
+    ("euro"     . 8364)))
+
 (defconst navi2ch-base64-begin-delimiter "----BEGIN BASE64----"
   "base64$B%3!<%I$NA0$KA^F~$9$k%G%j%_%?!#(B")
 (defconst navi2ch-base64-end-delimiter "----END BASE64----"
@@ -166,7 +253,12 @@ See also the function `defalias'."
     `(let ((,temp (default-file-modes)))
        (unwind-protect
 	   (progn
-	     (set-default-file-modes ,mode)
+	     (set-default-file-modes
+	      (navi2ch-ifxemacs
+		  (if (integerp ,mode)
+		      ,mode
+		    (char-to-int ,mode))
+		,mode))
 	     ,@body)
 	 (set-default-file-modes ,temp)))))
 
@@ -303,32 +395,37 @@ REGEXP $B$,8+$D$+$i$J$$>l9g!"(BSTRING $B$r$=$N$^$^JV$9!#(B"
 ;;       (sit-for 1))))
 
 (defsubst navi2ch-replace-html-tag-to-string (str)
-  (or (cdr (if case-fold-search
-	       (assoc-ignore-case str navi2ch-replace-html-tag-alist)
-	     (assoc str navi2ch-replace-html-tag-alist)))
-      (save-match-data
-	(let ((alist navi2ch-replace-html-tag-regexp-alist)
-	      elt value)
-	  (while alist
-	    (setq elt (car alist)
-		  alist (cdr alist))
-	    (when (string-match (car elt) str)
-	      (setq value (cdr elt)
-		    alist nil)))
-	  value))
-      ""))
+  (let ((ret
+	 (or (cdr (if case-fold-search
+		      (assoc-ignore-case str navi2ch-replace-html-tag-alist)
+		    (assoc str navi2ch-replace-html-tag-alist)))
+	     (save-match-data
+	       (let ((alist navi2ch-replace-html-tag-regexp-alist)
+		     elt value)
+		 (while alist
+		   (setq elt (car alist)
+			 alist (cdr alist))
+		   (when (string-match (car elt) str)
+		     (setq value (cdr elt)
+			   alist nil)))
+		 value))
+	     "")))
+    (if (functionp ret)
+	(funcall ret str)
+      ret)))
 
 (defsubst navi2ch-replace-html-tag (str)
   (let ((case-fold-search t))
     (navi2ch-replace-string navi2ch-replace-html-tag-regexp
 			    'navi2ch-replace-html-tag-to-string
-			    str t)))
+			    str t nil t)))
 
 (defsubst navi2ch-replace-html-tag-with-buffer ()
   (goto-char (point-min))
   (let ((case-fold-search t))
     (while (re-search-forward navi2ch-replace-html-tag-regexp nil t)
-      (replace-match (navi2ch-replace-html-tag-to-string (match-string 0))))))
+      (replace-match (navi2ch-replace-html-tag-to-string (match-string 0))
+		     nil t))))
 
 (defsubst navi2ch-replace-html-tag-with-temp-buffer (str)
   (with-temp-buffer
@@ -336,6 +433,42 @@ REGEXP $B$,8+$D$+$i$J$$>l9g!"(BSTRING $B$r$=$N$^$^JV$9!#(B"
     (navi2ch-replace-html-tag-with-buffer)
     (buffer-string)))
 
+(defun navi2ch-entity-reference-to-str (ref)
+  "$BJ8;z<BBN;2>H$r%G%3!<%I!#(B"
+  (save-match-data
+    (if (and navi2ch-article-decode-character-references
+	     (string-match "&\\([^;]+\\)" ref))
+	(let ((code (cdr (assoc (match-string 1 ref) navi2ch-entity-alist))))
+	  (or (and code (navi2ch-ucs-to-str code)) ref))
+      ref)))
+
+(defun navi2ch-numeric-reference-to-str (ref)
+  "$B?tCMJ8;z;2>H$r%G%3!<%I!#(B"
+  (save-match-data
+    (if (and navi2ch-article-decode-character-references
+	     (string-match "&#\\([^;]+\\)" ref))
+	(or (navi2ch-ucs-to-str (string-to-int (match-string 1 ref))) "$B".(B")
+      ref)))
+
+;; shut up byte-compile warnings
+(eval-when-compile
+  (autoload 'ucs-to-char "unicode")
+  (navi2ch-defalias-maybe 'unicode-to-char 'ignore)
+  (navi2ch-defalias-maybe 'decode-char 'ignore))
+
+(defun navi2ch-ucs-to-str (code)
+  (let ((c (cond
+	    ((featurep 'un-define)
+	     (ucs-to-char code))
+	    ((and (fboundp 'unicode-to-char)
+		  (subrp (symbol-function 'unicode-to-char)))
+	     (unicode-to-char code))
+	    (navi2ch-on-emacs21
+	     (decode-char 'ucs code)))))
+    (if (navi2ch-char-valid-p c)
+	(char-to-string c)
+      nil)))
+
 (defun navi2ch-read-char (&optional prompt)
   "PROMPT (non-nil $B$N>l9g(B) $B$rI=<($7$F(B `read-char' $B$r8F$S=P$9!#(B"
   (let ((cursor-in-echo-area t)
@@ -364,6 +497,20 @@ PROMPT) $B$rI=<($7$F:FEY(B `read-char' $B$r8F$V!#(B"
 	     (setq prompt (or retry-prompt prompt)))))
     c))
 
+(defun navi2ch-read-event (&optional prompt)
+  "PROMPT (non-nil $B$N>l9g(B) $B$rI=<($7$F(B event $B$rFI$`!#(B"
+  (let ((cursor-in-echo-area t)
+	(message-log-max nil)
+	e)
+    (if prompt
+	(message "%s" prompt))
+    (navi2ch-ifxemacs
+	(setq e (next-command-event nil prompt))
+      (setq e (read-event prompt)))
+    (if prompt
+	(message "%s%s" prompt (single-key-description e)))
+    e))
+
 (defun navi2ch-y-or-n-p (prompt &optional quit-symbol)
   (let* ((prompt (concat prompt "(y, n, or q) "))
 	 (c (navi2ch-read-char-with-retry
@@ -379,12 +526,12 @@ PROMPT) $B$rI=<($7$F:FEY(B `read-char' $B$r8F$V!#(B"
 
 (defsubst navi2ch-boundp (symbol)
   "SYMBOL $B$,%P%$%s%I$5$l$F$$$J$$;~$O(B nil $B$rJV$9!#(B
-bnoundp $B$H0c$$!"(BSYMBOL $B$,%P%$%s%I$5$l$F$$$k;~$O(B t $B$G$O$J$/%7%s%\%k$rJV$9!#(B"
+boundp $B$H0c$$!"(BSYMBOL $B$,%P%$%s%I$5$l$F$$$k;~$O(B t $B$G$O$J$/%7%s%\%k$rJV$9!#(B"
   (and (boundp symbol) symbol))
 
 (defsubst navi2ch-fboundp (symbol)
   "SYMBOL $B$,%P%$%s%I$5$l$F$$$J$$;~$O(B nil $B$rJV$9!#(B
-fbnoundp $B$H0c$$!"(BSYMBOL $B$,%P%$%s%I$5$l$F$$$k;~$O(B t $B$G$O$J$/%7%s%\%k$rJV$9!#(B"
+fboundp $B$H0c$$!"(BSYMBOL $B$,%P%$%s%I$5$l$F$$$k;~$O(B t $B$G$O$J$/%7%s%\%k$rJV$9!#(B"
   (and (fboundp symbol) symbol))
 
 (defun navi2ch-browse-url-internal (url &rest args)
@@ -449,6 +596,17 @@ return new alist whose car is the new pair and cdr is ALIST.
     (when point
       (1- point))))
 
+;(defun navi2ch-change-text-property (point prop value)
+;  (unless (get-text-property point prop)
+;    (error "POINT (%d) does not have property %s" point prop))
+;  (let ((start (if (or (= (point-min) point)
+;		       (not (eq (get-text-property (1- point) prop)
+;				(get-text-property point prop))))
+;		   point
+;		 (or (previous-single-property-change point prop) point)))
+;	(end (or (next-single-property-change point prop) point)))
+;    (put-text-property start end prop value)))
+
 (defun navi2ch-set-minor-mode (mode name map)
   (make-variable-buffer-local mode)
   (unless (assq mode minor-mode-alist)
@@ -642,8 +800,12 @@ base64$B%G%3!<%I$9$Y$-FbMF$,$J$$>l9g$O%(%i!<$K$J$k!#(B"
       (insert str))))
 
 (defun navi2ch-url-to-host (url)
-  (when (and url (string-match "http://\\([^/]+\\)" url))
-    (match-string 1 url)))
+  (when url
+    (cond
+     ((string-match "^http://\\([^/]+\\)" url)
+      (match-string 1 url))
+     ((string-match "^x-localbbs://" url)
+      "localhost"))))
 
 (defun navi2ch-read-string (prompt &optional initial-input history)
   (let ((minibuffer-allow-text-properties nil))
@@ -788,25 +950,26 @@ LOCKNAME $B$,@dBP%Q%9$G$O$J$$>l9g!"(BDIRECTORY $B$+$i$NAjBP%Q%9$H$7$F07$&!#(
     (delete-directory lockname))
   (not (file-exists-p lockname)))
 
-(defun navi2ch-line-beginning-position (&optional n)
-  "N - 1 $B9T@h$N9TF,$N>l=j$rJV$9!#(B"
-  (save-excursion
-    (beginning-of-line n)
-    (point)))
+(navi2ch-ifxemacs
+    (progn
+      (defalias 'navi2ch-line-beginning-position 'point-at-bol)
+      (defalias 'navi2ch-line-end-position 'point-at-eol))
+  (defalias 'navi2ch-line-beginning-position 'line-beginning-position)
+  (defalias 'navi2ch-line-end-position 'line-end-position))
 
-;; line-beginning-position $B$,;H$($k$J$i$=$C$A$r;H$&(B
-(if (fboundp 'line-beginning-position)
-    (defalias 'navi2ch-line-beginning-position 'line-beginning-position))
-
-(defun navi2ch-line-end-position (&optional n)
-  "N - 1 $B9T@h$N9TKv$N>l=j$rJV$9!#(B"
-  (save-excursion
-    (end-of-line n)
-    (point)))
-
-;; line-end-position $B$,;H$($k$J$i$=$C$A$r;H$&(B
-(if (fboundp 'line-end-position)
-    (defalias 'navi2ch-line-end-position 'line-end-position))
+(defun navi2ch-count-lines-file (file)
+  "$B$=$N%U%!%$%k$N9T?t$r?t$($k(B"
+  (with-temp-buffer
+    (insert-file-contents file)
+    (count-lines (point-min) (point-max))))
 
+(defun navi2ch-compare-times (t1 t2)
+  "t1 $B$,(B t2 $B$h$j?7$7$1$l$P(B t"
+  (cond
+   ((null (and t1 t2)) t)
+   ((> (nth 0 t1) (nth 0 t2)) t)
+   ((= (nth 0 t1) (nth 0 t2)) (> (nth 1 t1) (nth 1 t2)))
+   (t nil)))
+    
 (run-hooks 'navi2ch-util-load-hook)
 ;;; navi2ch-util.el ends here
