@@ -27,7 +27,7 @@
 ;;; Code:
 (provide 'navi2ch-vars)
 (defvar navi2ch-vars-ident
-  "$Id: navi2ch-vars.el,v 1.77 2002/07/26 13:55:11 nanashi Exp $")
+  "$Id: navi2ch-vars.el,v 1.99 2002/11/16 18:01:04 mami Exp $")
 
 (defconst navi2ch-on-xemacs (featurep 'xemacs))
 (defconst navi2ch-on-emacs21 (and (not navi2ch-on-xemacs)
@@ -291,6 +291,11 @@ nil $B$J$i(B expire $B$7$J$$!#(B"
   :type 'integer
   :group 'navi2ch-board)
 
+(defcustom navi2ch-bm-number-width 3
+  "*$B3F%9%l$N%9%lHV9fMs$NI}!#(B"
+  :type 'integer
+  :group 'navi2ch-board)
+
 (defcustom navi2ch-bm-mark-and-move t
   "*$B%^!<%/$7$?$"$H$N%]%$%s%?$NF0:n!#(B
 nil $B$J$i0\F0$7$J$$(B
@@ -331,6 +336,12 @@ non-nil $B$J$i2<$K0\F0$9$k(B
   :type 'file
   :group 'navi2ch-board)
 
+(defcustom navi2ch-bookmark-remember-order-after-sort nil
+  "*bookmark $B%b!<%I$G(B sort $B8e$N%9%lJB$S=g$r5-21$9$k$+$I$&$+!#(B
+non-nil $B$J$i$P5-21$9$k!#(B"
+  :type 'boolean
+  :group 'navi2ch-board)
+
 (defcustom navi2ch-history-file (concat
                                  (file-name-as-directory navi2ch-directory)
                                  "history.txt")
@@ -355,6 +366,35 @@ non-nil $B$J$i$P(B expire $B$9$k!#(B"
 		 (const :tag "$B%U%'%$%9$r;HMQ$7$J$$(B" nil))
   :group 'navi2ch-board)
 
+(defcustom navi2ch-bm-sort-by-state-order
+  '(("+U" . 0)
+    ("+V" . 1)
+    ("+C" . 2)
+    ("+ " . 3)
+    (" U" . 4)
+    (" V" . 5)
+    (" C" . 6)
+    ("  " . 7)
+    ("=U" . 8)
+    ("=V" . 9)
+    ("=C" . 10)
+    ("= " . 11))
+  "*$B>uBV$G%=!<%H$9$k$H$-$N=g=x$r7h$a$k%j%9%H!#(B"
+  :type '(list (cons (const :tag "$B>uBV(B +U" "+U") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B +V" "+V") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B +C" "+C") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B + " "+ ") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B  U" " U") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B  V" " V") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B  C" " C") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B   " "  ") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B =U" "=U") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B =V" "=V") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B =C" "=C") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B = " "= ") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B   " "  ") (number :tag "$B=gHV(B")))
+  :group 'navi2ch-board)
+
 ;;; article variables
 (defcustom navi2ch-article-aadisplay-program
   (if (eq window-system 'w32)
@@ -419,6 +459,17 @@ ask $B$J$iL@<(E*$K0\F0$9$k;~0J30$J$i<ALd$9$k(B
 		 (const :tag "$B0\F0$7$J$$(B" nil))
   :group 'navi2ch-article)
 
+(defcustom navi2ch-article-through-ask-function
+  #'navi2ch-article-through-ask-y-or-n-p
+  "*$B<!$N%9%l%C%I$K0\F0$9$k$H$-$N3NG'$K;HMQ$9$k4X?t!#(B"
+  :type '(choice (const :tag "y or n $B$G3NG'(B"
+			navi2ch-article-through-ask-y-or-n-p)
+		 (const :tag "n $B$^$?$O(B p $B$G3NG'(B"
+			navi2ch-article-through-ask-n-or-p-p)
+		 (const :tag "$BD>A0$N%3%^%s%I$HF1$8$+$G3NG'(B"
+			navi2ch-article-through-ask-last-command-p))
+  :group 'navi2ch-article)
+
 (defcustom navi2ch-article-parse-field-list '(data name mail)
   "*$B%a%C%;!<%8$N%U%#!<%k%I$N$&$A!"%Q!<%:BP>]$K$9$k$b$N$N%j%9%H!#(B
 $BCY$/$F$b$$$$$s$J$i(B '(data mail name) $B$H$+$9$k$H$$$$$+$b(B"
@@ -472,14 +523,24 @@ ask $B$J$iL@<(E*$K0\F0$9$k;~0J30$J$i<ALd$9$k(B
   :type 'regexp
   :group 'navi2ch-article)
 
-(defcustom navi2ch-article-number-regexp
-  "[>$B!d(B<$B!c(B][>$B!d(B<$B!c(B ]*\\(\\([0-9$B#0(B-$B#9(B]+,\\)*[0-9$B#0(B-$B#9(B]+\\(-[0-9$B#0(B-$B#9(B]+\\)?\\)"
+(defcustom navi2ch-article-number-prefix-regexp "[>$B!d(B<$B!c(B][>$B!d(B<$B!c(B]* *"
   "*$BF1$8%9%lFb$X$N%j%s%/$rI=$o$9@55,I=8=!#(B"
   :type 'regexp
   :group 'navi2ch-article)
 
+(defcustom navi2ch-article-number-separator-regexp " *, *"
+  "*$BF1$8%9%lFb$X$N%j%s%/$N?t;z$r6h@Z$kJ8;zNs$rI=$o$9@55,I=8=!#(B"
+  :type 'regexp
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-number-number-regexp
+  "\\([0-9$B#0(B-$B#9(B]+\\(-[0-9$B#0(B-$B#9(B]+\\)?\\)"
+  "*$BF1$8%9%lFb$X$N%j%s%/$N?t;z$rI=$o$9@55,I=8=!#(B"
+  :type 'regexp
+  :group 'navi2ch-article)
+
 (defcustom navi2ch-article-url-regexp
-  "h?ttps?://\\([-a-zA-Z0-9_=?#$@~`%&*+|\\/.,:]+\\)"
+  "\\(h?ttps?\\|x-localbbs\\)\\(://[-a-zA-Z0-9_=?#$@~`%&*+|\\/.,:]+\\)"
   "*$B%l%9$N%F%-%9%H$N$&$A(B URL $B$H$_$J$9ItJ,$N@55,I=8=!#(B"
   :type 'regexp
   :group 'navi2ch-article)
@@ -591,6 +652,85 @@ nil $B$N>l9g$OF1$8%9%l$NFbMF$N$_$rF@$k!#(B"
   :type 'boolean
   :group 'navi2ch-article)
 
+(defcustom navi2ch-article-dispweek nil
+  "* non-nil$B$J$i$P(BYY/MM/DD$B$NF|IU$KMKF|I=<($7!$G/$r(BYYYY$BI=5-$K$9$k(B"
+  :type 'boolean
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-decode-character-references t
+  "*non-nil$B$J$i!"?tCMJ8;z;2>H!"J8;z<BBN;2>H$NI=<($r;n$_$k!#(B
+GNU Emacs 21, XEmacs 21.5 $B0J9_$G$"$l$P%G%U%)%k%H$GI=<($G$-$^$9$,!"(B
+$B$=$l0JA0$N(BEmacsen$B$G$O(BMule-UCS$B$,I,MW$G$9!#(B (require 'un-define) $B$7$F$M!#(B"
+  :type 'boolean
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-message-filter-list
+  '(navi2ch-article-message-filter-by-name
+    navi2ch-article-message-filter-by-id
+    navi2ch-article-message-filter-by-message)
+  "*$B%l%9$r%U%#%k%?$9$k$?$a$N4X?t$N%j%9%H!#(B
+$B%j%9%H$N(B member $B$H$J$k4X?t$H$7$F$O!"(B
+$B%l%9$N(B alist $B$r0z$-?t$K<h$j!"(B
+$B%U%#%k%?8e$NJ8;zNs$b$7$/$O(B 'hide $B$+(B 'important $B$rJV$9$b$N$r;XDj$9$k!#(B
+
+$B2<5-$OL>A0Ms$,!V$[$2!W$N$H$-$K!V$"$\$\!<$s!W$KCV49$9$k$?$a$N(B
+($B$A$g$C$H>iD9$J(B) $B4X?t$NNc!#(B
+
+(defun my-navi2ch-article-message-filter-hoge (alist)
+  (let ((name (cdr (assq 'name alist)))
+	(mail (cdr (assq 'mail alist)))
+	(date (cdr (assq 'date alist)))
+	(message (cdr (assq 'data alist))))
+    (if (equal name \"$B$[$2(B\")
+	\"$B$"$\$\!<$s(B\"
+      nil))"
+  :type '(repeat function)
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-message-filter-by-name-alist nil
+  "*$B%l%9$r%U%#%k%?$9$kL>A0$H!"CV$-49$($kJ8;zNs$N(B alist$B!#(B
+$BCV$-49$($kJ8;zNs$NBe$o$j$K(B hide $B$d(B important $B$r;XDj$G$-$k!#(B
+$BNc$($P2<5-$NCM$r@_Dj$9$k$H!"(B
+$BL>A0Ms$K!V$U$,!W$,4^$^$l$F$$$k$H%l%9$,!V$"$\$\!<$s!W$KCV$-49$o$j!"(B
+$BL>A0Ms$K!V%[%2!W$,4^$^$l$F$$$k$H%l%9$,1#$5$l$k!#(B
+
+'((\"$B$U$,(B\" . \"$B$"$\$\!<$s(B\")
+  (\"$B%[%2(B\" . hide))"
+  :type '(repeat (cons (string :tag "$BL>A0Ms(B")
+		       (string :tag "$BCV498e(B")))
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-message-filter-by-message-alist nil
+  "*$B%l%9$r%U%#%k%?$9$k$?$a$N%l%9K\J8Cf$NJ8;zNs$H!"CV$-49$($kJ8;zNs$N(B alist$B!#(B
+$BCV$-49$($kJ8;zNs$NBe$o$j$K(B hide $B$d(B important $B$r;XDj$G$-$k!#(B
+$BNc$($P2<5-$NCM$r@_Dj$9$k$H!"(B
+$B%l%9K\J8Cf$K!V$U$,!W$,4^$^$l$F$$$k$H%l%9$,!V$"$\$\!<$s!W$KCV$-49$o$j!"(B
+$B%l%9K\J8Cf$K!V%[%2!W$,4^$^$l$F$$$k$H%l%9$,1#$5$l$k!#(B
+
+'((\"$B$U$,(B\" . \"$B$"$\$\!<$s(B\")
+  (\"$B%[%2(B\" . hide))"
+  :type '(repeat (cons (string :tag "$BK\J8Cf(B")
+		       (string :tag "$BCV498e(B")))
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-message-filter-by-id-alist nil
+  "*$B%l%9$r%U%#%k%?$9$k$?$a$N(B ID $BCf$NJ8;zNs$H!"CV$-49$($kJ8;zNs$N(B alist$B!#(B
+$BCV$-49$($kJ8;zNs$NBe$o$j$K(B hide $B$d(B important $B$r;XDj$G$-$k!#(B
+$BNc$($P2<5-$NCM$r@_Dj$9$k$H!"(B
+ID $B$,!V(BFUga1234$B!W$@$H%l%9$,!V$"$\$\!<$s!W$KCV$-49$o$j!"(B
+ID $B$,!V(BhoGE0987$B!W$@$H%l%9$,1#$5$l$k!#(B
+
+'((\"FUga1234\" . \"$B$"$\$\!<$s(B\")
+  (\"hoGE0987\" . hide))"
+  :type '(repeat (cons (string :tag "ID")
+		       (string :tag "$BCV498e(B")))
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-save-info-wrapper-func nil
+  "*navi2ch-article-save-info $BCf$G(B article $B$N(B wrapper $B$H$7$F;H$&4X?t!#(B"
+  :type 'function
+  :group 'navi2ch-article)
+
 ;;; message variables
 (defcustom navi2ch-message-user-name
   (cond ((featurep 'xemacs) "$BL>L5$7$5$s!w#X#E#m#a#c#s(B")
@@ -648,8 +788,13 @@ nil $B$J$i!"=q$-$+$1$rGK4~$7$F$$$$$+Ld$$9g$o$;$k!#(B
   :type 'integer
   :group 'navi2ch-message)
 
+(defcustom navi2ch-message-retry-wait-time 2
+  "*$B%l%9Aw?.$r:F;n9T$9$k$H$-$NBT$A;~4V(B($BIC(B)$B!#(B"
+  :type 'integer
+  :group 'navi2ch-message)
+
 (defcustom navi2ch-message-remember-user-name t
-  "*non-nil$B$J$i!"Aw$C$?%l%9$NL>A0Mw$r3P$($F$*$/!#(B
+  "*non-nil$B$J$i!"Aw$C$?%l%9$NL>A0Mw$H%a!<%kMs$r3P$($F$*$/!#(B
 $BF1$8%9%l$G<!$K%l%9$9$k$H$-$O!"$=$l$,%G%U%)%k%H$NL>A0$K$J$k!#(B"
   :type 'boolean
   :group 'navi2ch-message)
@@ -675,6 +820,7 @@ nil $B$J$i!"=q$-$+$1$rGK4~$7$F$$$$$+Ld$$9g$o$;$k!#(B
     ("b" . "$B!3(B(`$B'%!-(B)(II(B")
     ("B" . "((($B!((I_$B'%(I_(B))(I6^86^8L^YL^Y(B")
     ("f" . "( $B!-(B_$B!5(B`)(IL0](B")
+    ("e" . "($B"?'U(B`)")
     ("F" . "($B!-!<!.(B)")
     ("g" . "((I_$B'%(I_(B)(I:^Y'(B")
     ("G" . "(I6^$B(,(,(B((I_$B'%(I_(B;)$B(,(,(I?(B!")
@@ -692,6 +838,7 @@ nil $B$J$i!"=q$-$+$1$rGK4~$7$F$$$$$+Ld$$9g$o$;$k!#(B
     ("p" . "$B!J!!(I_$B'U(I_$B!K(IN_60](B")
     ("s" . "$B&2!J(I_$B'U(I_(Blll$B!K(I6^0](B")
     ("S" . "($B!-(I%$B&X(I%(B`)(I<.N^0](B")
+    ("t" . "y=(I0(B( (I_$B'U(I_(B)(I%$B"h(B.$B!!(I@0](B")
     ("u" . "((I_$B'U(I_(B)(I3O0(B")
     ("U" . "(-$B!2(B-)(I3B@^(B"))
   "AA $B$rF~NO$9$k$H$-$N%-!<%P%$%s%I$H(B AA $B$N(B alist$B!#(B
@@ -726,6 +873,30 @@ SPC$B!"(BC-l$B!"(BC-g$B!"(BC-v$B$O%j%9%HI=<($N:]$K;HMQ$5$l$k$N$G%-!<$K$O;
   :type 'boolean
   :group 'navi2ch-message)
 
+(defcustom navi2ch-message-save-sendlog nil
+  "*non-nil $B$J$i!"Aw?.$7$?%l%9$N95$($r$H$k!#(B
+Emacs $B$N%+%9%?%^%$%:%$%s%?!<%U%'%$%9$r;H$C$F$3$NCM$r(B non-nil $B$K$7$?>l9g!"(B
+Navi2ch$B%+%F%4%j$K!VAw?.95$(!WHD$,<+F0E*$KDI2C$5$l$^$9!#(B
+`navi2ch-init-file'$B$K(B
+  (setq navi2ch-message-save-sendlog t)
+$B$H=q$$$?>l9g$O!"$5$i$K(B
+  (add-to-list 'navi2ch-list-navi2ch-category-alist
+               navi2ch-message-sendlog-board)
+$B$rDI2C$7!"(BNavi2ch$B%+%F%4%j$K!VAw?.95$(!WHD$rDI2C$7$F2<$5$$!#(B"
+  :type 'boolean
+  :group 'navi2ch-message
+  :set (lambda (symbol value)
+	 (when value
+	   (eval-after-load "navi2ch"
+	     '(add-to-list 'navi2ch-list-navi2ch-category-alist
+			   navi2ch-message-sendlog-board)))
+	 (set-default symbol value)))
+
+(defcustom navi2ch-message-sendlog-subject "$BAw?.95$((B"
+  "*$BAw?.$7$?%l%9$rJ]B8$9$k%9%l$N%?%$%H%k!#(B"
+  :type 'string
+  :group 'navi2ch-message)
+
 ;; net variables
 (defcustom navi2ch-net-http-proxy
   (if (string= (getenv "HTTP_PROXY") "")
@@ -934,6 +1105,11 @@ a symbol `bitmap', `xbm' or `xpm' in order to force the image format."
 (defvar navi2ch-popup-article-exit-hook nil)
 (defvar navi2ch-head-mode-hook nil)
 (defvar navi2ch-head-exit-hook nil)
+(defvar navi2ch-mona-setup-hook nil)
+(defvar navi2ch-mona-undo-setup-hook nil)
+(defvar navi2ch-directory-mode-hook nil)
+(defvar navi2ch-directory-exit-hook nil)
+
 ;; load hooks
 (defvar navi2ch-article-load-hook nil)
 (defvar navi2ch-articles-load-hook nil)
@@ -952,6 +1128,7 @@ a symbol `bitmap', `xbm' or `xpm' in order to force the image format."
 (defvar navi2ch-util-load-hook nil)
 (defvar navi2ch-vars-load-hook nil)
 (defvar navi2ch-load-hook nil)
+(defvar navi2ch-directory-load-hook nil)
 
 ;;; errors symbols
 (put 'navi2ch-update-failed 'error-conditions '(error navi2ch-errors navi2ch-update-failed))
