@@ -23,7 +23,7 @@
 ;;; Code:
 (provide 'navi2ch)
 (defvar navi2ch-ident
-  "$Id: navi2ch.el,v 1.50 2002/07/26 13:55:11 nanashi Exp $")
+  "$Id: navi2ch.el,v 1.53 2002/10/19 13:52:52 tabmore Exp $")
 
 (eval-when-compile (require 'cl))
 
@@ -52,6 +52,8 @@
 (require 'navi2ch-machibbs)
 (require 'navi2ch-multibbs)
 (require 'navi2ch-mona)
+(require 'navi2ch-oyster)
+(require 'navi2ch-localfile)
 
 (defgroup navi2ch nil
   "Navigator for 2ch."
@@ -367,11 +369,9 @@ DISPLAY $B$,(B `article' $B$N$H$-$O(B article $B$rI=<($9$kMQ$KJ,3d$9$k!#(B
     (and (or (member host navi2ch-2ch-host-list)
 	     (let (list)
 	       (setq list
-		     (mapcar
-		      (lambda (x)
-			(navi2ch-url-to-host (cdr (assq 'uri x))))
-		      (navi2ch-list-get-board-name-list
-		       navi2ch-list-category-list)))
+		     (mapcar (lambda (x)
+			       (navi2ch-url-to-host (cdr (assq 'uri x))))
+			     navi2ch-list-board-name-list))
 	       (member host list)))
 	 (or (navi2ch-article-url-to-article url)
 	     (navi2ch-board-url-to-board url))
